/*
** Module   : ButtonWidget.java
** Abstract : server side button widget implementation
**
** Copyright (c) 2005-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 NVS 20051026   @23482 Created initial version.
** 002 GES 20060114   @23907 Removed totally useless screen-value attr.
** 003 GES 20060803   @28367 Some cleanup and refactoring.
** 004 GES 20061116   @31247 Exposed button-specific flags on the server
**                           side.
** 005 GES 20080320   @37582 Added TYPE attribute.
** 006 SIY 20101001          Added more button options.
** 007 SIY 20101004          Added IMAGE property.
** 008 SIY 20101006          Added LOAD-IMAGE method.
** 009 SIY 20111101          Updated imports.
** 010 CA  20130221          Refactored usage of TYPE attribute.
** 011 CA  20130222          Added NPE protection for cases when a widget is used before is 
**                           attached to a frame (after a CREATE WIDGET statement is used).
** 012 CA  20130927          Added LegacyResource annotation, to determine resource type.
** 013 CA  20131115          Changes related to properly instantiate dynamic resources.
** 014 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 015 EVL 20141222          Moving tooltip related methods to the base class to be used for every
**                           widget that needs tooltip feature.
** 016 EVL 20141226          Adding support for methods LOAD-IMAGE-UP, LOAD-IMAGE-DOWN,
**                           LOAD-IMAGE-INSENSITIVE.
** 017 EVL 20150115          Rework for image support implementation.
** 018 EVL 20150121          Implementing pending image loading. 
*/ 

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;

/**
 * Server side button widget implementation.
 */
@LegacyResource(resource = LegacyResource.BUTTON)
public class ButtonWidget
extends ControlEntity<ButtonConfig>
{
   /**
    * Default constructor.
    */
   public ButtonWidget()
   {
      this(false);
   }
   
   /**
    * Default constructor.
    * 
    * @param   dynamic
    *          Flag indicating if this is a static or dynamic resource.
    */
   public ButtonWidget(boolean dynamic)
   {
      super(dynamic, new ButtonConfig());
   }
   
   /**
    * Set the flag for whether or not this is a default button.
    * 
    * @param    isDefault
    *           <code>true</code> to make it default or <code>false</code>
    *           to make it an ordinary button.
    */
   public void setDefault(boolean isDefault)
   {
      config.isDefault = isDefault;
      pushScreenDefinition();
   }

   /**
    * Set the flag for the AUTO-GO attribute.
    * 
    * @param    autoGo
    *           The new value for the attribute.
    */
   public void setAutoGo(boolean autoGo)
   {
      config.autoGo = autoGo;
      pushScreenDefinition();
   }

   /**
    * Set the flag for the AUTO-ENDKEY attribute.
    * 
    * @param    autoEndKey
    *           The new value for the attribute.
    */
   public void setAutoEndKey(boolean autoEndKey)
   {
      config.autoEndKey = autoEndKey;
      pushScreenDefinition();
   }

   /**
    * Returns the name of the image loaded by LOAD-IMAGE. Corresponds Progress
    * IMAGE attribute
    *
    * @return the name of the image loaded by LOAD-IMAGE. 
    */
   public character getImage()
   {
      return new character(config.image);
   }

   /**
    * Set button image file name.
    * 
    * @param   name
    *          Image file name.
    */
   public void setImage(String name)
   {
      loadImage(name);
   }
   
   /**
    * Set button up image file name.
    * 
    * @param   name
    *          Image file name.
    */
   public void setImageUp(String name)
   {
      loadImageUp(name);
   }

   /**
    * Set button down image file name.
    * 
    * @param   name
    *          Image file name.
    */
   public void setImageDown(String name)
   {
      loadImageDown(name);
   }
   
   /**
    * Set button insensitive image file name.
    * 
    * @param   name
    *          Image file name.
    */
   public void setImageIns(String name)
   {
      loadImageInsensitive(name);
   }
   
   /**
    * Set NO-FOCUS option.
    * 
    * @param   flag
    *          New value for the option.
    */
   public void setNoFocus(boolean flag)
   {
      config.noFocus = flag;
      pushScreenDefinition();
   }

   /**
    * Set NO-CONVERT-3D-COLORS option.
    * 
    * @param   flag
    *          New value for the option.
    */
   public void setNoConvert3D(boolean flag)
   {
      config.noConvert3D = flag;
      pushScreenDefinition();
   }

   /**
    * Load button image from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   public logical loadImage(String name)
   {
      if (loadImageInt(name, ButtonConfig.IMAGE_UP))
      {
         // only if image was loaded we need to change it's name in config
         config.image = name;
         pushScreenDefinition();
         return new logical(true);
      }
      else
      {
         return new logical(false);
      }
   }

   /**
    * Load button image from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   public logical loadImage(character name)
   {
      return name != null && !name.isUnknown() ? loadImage(name.toStringMessage())
                                               : new logical(false);
   }

   /**
    * Load button image for normal state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   public logical loadImageUp(String name)
   {
      if (loadImageInt(name, ButtonConfig.IMAGE_UP))
      {
         // only if image was loaded we need to change it's name in config
         config.imageUp = name;
         pushScreenDefinition();
         return new logical(true);
      }
      else
      {
         return new logical(false);
      }
   }

   /**
    * Load button image for normal state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   public logical loadImageUp(character name)
   {
      return name != null && !name.isUnknown() ? loadImageUp(name.toStringMessage())
                                               : new logical(false);
   }

   /**
    * Load button image for pushed state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   public logical loadImageDown(String name)
   {
      if (loadImageInt(name, ButtonConfig.IMAGE_DOWN))
      {
         // only if image was loaded we need to change it's name in config
         config.imageDown = name;
         pushScreenDefinition();
         return new logical(true);
      }
      else
      {
         return new logical(false);
      }
   }

   /**
    * Load button image for pushed state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   public logical loadImageDown(character name)
   {
      return name != null && !name.isUnknown() ? loadImageDown(name.toStringMessage())
                                               : new logical(false); 
   }

   /**
    * Load button image for disabled state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   public logical loadImageInsensitive(String name)
   {
      if (loadImageInt(name, ButtonConfig.IMAGE_INS))
      {
         // only if image was loaded we need to change it's name in config
         config.imageIns = name;
         pushScreenDefinition();
         return new logical(true);
      }
      else
      {
         return new logical(false);
      }
   }

   /**
    * Load button image for disabled state from specified file name.
    * 
    * @param   name
    *          Image file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   public logical loadImageInsensitive(character name)
   {
      return name != null && !name.isUnknown() ? loadImageInsensitive(name.toStringMessage())
                                               : new logical(false); 
   }

   /**
    * Load image for button.  Internal version.
    * 
    * @param    imageSrc
    *           Image file name to load.
    * @param    type
    *           Image type to load.  Up, Down or Insensitive.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   private boolean loadImageInt(String imageSrc, int type)
   {
      // logical terminal will try to load image from application jar, then ask the
      // client to load from local file system in cas of failure
      return LogicalTerminal.setImage(config.id.getId(), type, imageSrc); 
   }
}
